@@ -4,8 +4,10 @@
 #endif
 
 #include "da.h"
-#include "capplet-widget.h"
 #include <signal.h>
+#include <gconf/gconf-client.h>
+
+GtkWidget *preview_control;
 
 static gboolean   ignore_change = FALSE;
 
@@ -15,10 +17,10 @@ static GtkWidget *capplet_widget;
 static GtkWidget *theme_list;
 static GtkWidget *auto_preview;
 
-static GtkWidget *current_theme = NULL;
-static GtkWidget *current_global_theme = NULL;
-static GtkWidget *initial_theme = NULL;
-static GtkWidget *last_theme = NULL;
+static ThemeEntry *current_theme = NULL;
+static ThemeEntry *current_global_theme = NULL;
+static ThemeEntry *initial_theme = NULL;
+static ThemeEntry *last_theme = NULL;
 static GtkWidget *font_sel;
 static GtkWidget *font_cbox;
 static gboolean initial_preview;
@@ -36,9 +38,23 @@ click_ok(GtkWidget *widget, gpointer data);
 static void
 click_revert(GtkWidget *widget, gpointer data);
 static void
-click_entry(GtkWidget *widget, gpointer data);
+click_entry(GtkWidget *clist, gint row, gint col, GdkEvent *event,
+	    gpointer data);
 static void
-delete_entry(GtkWidget *widget, gpointer data);
+state_changed (gboolean state);
+static void
+response_cb (GtkDialog *dialog, GtkResponseType response, gpointer data);
+
+void
+send_reread(void)
+{
+	if (!(current_theme && current_theme->rc))
+		return;
+
+	bonobo_widget_set_property (BONOBO_WIDGET (preview_control),
+				    "theme", TC_CORBA_string,
+				    gtkrc_tmp, NULL);
+}
 
 static void
 auto_callback (GtkWidget *widget, gpointer data)
@@ -46,7 +62,7 @@ auto_callback (GtkWidget *widget, gpointer data)
   if (ignore_change == FALSE) {
 	  if (GTK_TOGGLE_BUTTON (auto_preview)->active)
 		  click_preview (widget,NULL);
-	  capplet_widget_state_changed(CAPPLET_WIDGET (capplet_widget), TRUE);
+	  state_changed (TRUE);
   }
   
 }
@@ -54,7 +70,7 @@ static void
 font_callback (GtkWidget *widget, gchar *font, gpointer data)
 {
   if (ignore_change == FALSE) {
-	capplet_widget_state_changed(CAPPLET_WIDGET (capplet_widget), TRUE);
+	state_changed (TRUE);
 	if (GTK_TOGGLE_BUTTON (auto_preview)->active)
 	  click_preview (widget,NULL);
   }	
@@ -63,7 +79,7 @@ static void
 use_theme_font_callback (GtkWidget *widget, gpointer data)
 {
   if (ignore_change == FALSE) {
-	capplet_widget_state_changed(CAPPLET_WIDGET (capplet_widget), TRUE);
+	state_changed (TRUE);
 	if (GTK_TOGGLE_BUTTON (auto_preview)->active)
 		click_preview (widget,NULL);
 	if (!GTK_TOGGLE_BUTTON (font_cbox)->active)
@@ -127,6 +143,7 @@ install_theme_callback (GtkWidget *widget, gpointer data)
   /* We'd like to set a transient_for hint here, but it isn't
    * worth the bother, since our parent window isn't in this process
    */
+#if 0
   if ( gnome_preferences_get_dialog_centered() ) {
 	  /* User wants us to center over parent */
 
@@ -150,6 +167,7 @@ install_theme_callback (GtkWidget *widget, gpointer data)
 					   dialog_x, dialog_y);
 	  }
   }
+#endif
   gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (install_theme_file_sel)
 				  ->ok_button), "clicked",
 		      (GtkSignalFunc) browse_dialog_ok,
@@ -169,16 +187,6 @@ install_theme_callback (GtkWidget *widget, gpointer data)
 
 }
 
-static gint
-delete_capplet (GtkWidget *widget, GdkEvent *event, gpointer data)
-{
-  /* We don't want the toplevel window destroyed until
-   * our child exits.
-   */
-  close(prog_fd);
-  return FALSE;
-}
-
 GtkWidget *
 make_main(void)
 {
@@ -186,10 +194,17 @@ make_main(void)
   GtkWidget *box, *hbox, *hbox2, *vbox;
   GtkWidget *frame, *button;
   GtkWidget *button_vbox;
-  gboolean default_used;
+  BonoboUIContainer *container;
+  GConfClient *client = gconf_client_get_default ();
   
-  capplet_widget = capplet_widget_new();
-  gtk_container_set_border_width(GTK_CONTAINER(capplet_widget), 5);
+  capplet_widget = gtk_dialog_new_with_buttons (_("Gtk+ Theme Selector"),
+					 NULL, -1,
+					 GTK_STOCK_HELP, GTK_RESPONSE_HELP,
+					 GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
+					 GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+					 NULL);
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (capplet_widget),
+		  		     GTK_RESPONSE_APPLY, FALSE);
 
   box = gtk_vbox_new(FALSE, GNOME_PAD);
   hbox = gtk_hbox_new(TRUE, GNOME_PAD);
@@ -202,11 +217,14 @@ make_main(void)
 
   /* List of available themes
    */
-  theme_list = gtk_list_new();
-  gtk_list_set_selection_mode(GTK_LIST(theme_list), GTK_SELECTION_BROWSE);
+  theme_list = gtk_clist_new (1);
+  gtk_signal_connect (GTK_OBJECT (theme_list), "select_row", click_entry,
+		      NULL);
+  gtk_clist_set_selection_mode(GTK_CLIST(theme_list), GTK_SELECTION_BROWSE);
   sw = gtk_scrolled_window_new(NULL, NULL);
-  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), theme_list);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC,
+  gtk_container_add (GTK_CONTAINER(sw), theme_list);
+  /* Mysterious allocation bug keeps shrinking hscrollbar during browse */
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER,
 				 GTK_POLICY_AUTOMATIC);
   gtk_widget_set_usize (sw, 120, -1);
 
@@ -221,7 +239,7 @@ make_main(void)
   label = gtk_label_new (_("Auto\nPreview"));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   auto_preview = gtk_check_button_new ();
-  initial_preview = gnome_config_get_bool ("/theme-switcher-capplet/settings/auto=TRUE");
+  initial_preview = gconf_client_get_bool (client, "/apps/gtk-theme-switcher/auto", NULL);
   gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (auto_preview),
 			       initial_preview);
   gtk_signal_connect (GTK_OBJECT (auto_preview), "toggled", GTK_SIGNAL_FUNC (auto_callback), NULL);
@@ -241,8 +259,7 @@ make_main(void)
   font_sel = gnome_font_picker_new ();
   gnome_font_picker_set_mode (GNOME_FONT_PICKER (font_sel),
 			      GNOME_FONT_PICKER_MODE_FONT_INFO);
-  initial_font = gnome_config_get_string_with_default ("/theme-switcher-capplet/settings/font",&default_used);
-
+  initial_font = gconf_client_get_string (client, "/apps/gtk-theme-switcher/font", NULL);
 
   if (initial_font == NULL) {
 
@@ -256,7 +273,7 @@ make_main(void)
       /*         for now we just assume default gtk font */
       initial_font = g_strdup(_("-adobe-helvetica-medium-r-normal--*-120-*-*-*-*-*-*"));
     } else {
-      initial_font = style->rc_style->font_name;
+      initial_font = pango_font_description_to_string (style->rc_style->font_desc);
     }
   }
 
@@ -274,7 +291,7 @@ make_main(void)
   gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
   gtk_container_add (GTK_CONTAINER (frame), vbox);
   font_cbox = gtk_check_button_new_with_label (_("Use custom font."));
-  initial_font_cbox = gnome_config_get_bool ("/theme-switcher-capplet/settings/use_theme_font=FALSE");
+  initial_font_cbox = gconf_client_get_bool (client, "/apps/gtk-theme-switcher/use_theme_font", NULL);
   gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (font_cbox),
 			       initial_font_cbox);
   gtk_signal_connect (GTK_OBJECT (font_cbox),
@@ -306,26 +323,21 @@ make_main(void)
   frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN); 
   gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);
-  preview_socket = gtk_socket_new();
-  gtk_container_add(GTK_CONTAINER(frame), preview_socket);
+  container = bonobo_ui_container_new ();
+  preview_control = bonobo_widget_new_control ("OAFIID:GNOME_Theme_Preview",
+		  			       BONOBO_OBJREF (container));
+  bonobo_object_unref (BONOBO_OBJECT (container));
+
+  gtk_container_add(GTK_CONTAINER(frame), preview_control);
   update_theme_entries(theme_list);
 
-  gtk_signal_connect (GTK_OBJECT (capplet_widget), "help",
-		      GTK_SIGNAL_FUNC (click_help), NULL);
-  gtk_signal_connect (GTK_OBJECT (capplet_widget), "try",
-		      GTK_SIGNAL_FUNC (click_try), NULL);
-  gtk_signal_connect (GTK_OBJECT (capplet_widget), "ok",
-		      GTK_SIGNAL_FUNC (click_ok), NULL);
-  gtk_signal_connect (GTK_OBJECT (capplet_widget), "revert",
-		      GTK_SIGNAL_FUNC (click_revert), NULL);
-  gtk_signal_connect (GTK_OBJECT (capplet_widget), "cancel",
-		      GTK_SIGNAL_FUNC (click_revert), NULL);
-  gtk_signal_connect (GTK_OBJECT (capplet_widget), "delete_event",
-		      GTK_SIGNAL_FUNC (delete_capplet), NULL);
-  gtk_container_add (GTK_CONTAINER (capplet_widget), box);
+  g_signal_connect (G_OBJECT (capplet_widget), "response", response_cb, NULL);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (capplet_widget)->vbox), box,
+		      TRUE, TRUE, 0);
 
   last_theme = NULL;
-  
+ 
+  g_object_unref (G_OBJECT (client));
   return capplet_widget;
 }
 
@@ -340,7 +352,7 @@ click_preview(GtkWidget *widget, gpointer data)
   if (!current_theme) {
     return;
   }
-  rc = (gchar *)gtk_object_get_data(GTK_OBJECT(current_theme), "rc");
+  rc = current_theme->rc;
   if (GTK_TOGGLE_BUTTON (font_cbox)->active)
     test_theme(rc,
 	      gnome_font_picker_get_font_name (GNOME_FONT_PICKER (font_sel)));
@@ -354,22 +366,7 @@ click_preview(GtkWidget *widget, gpointer data)
 static void
 click_help(GtkWidget *widget, gpointer data)
 {
-  gchar *tmp;
-
-  tmp = gnome_help_file_find_file ("users-guide", "gccdesktop.html#GCCTHEME");
-  if (tmp) {
-    gnome_help_goto(0, tmp);
-    g_free(tmp);
-  } else {
-    GtkWidget *mbox;
-
-    mbox = gnome_message_box_new(_("No help is available/installed for these settings. Please make sure you\nhave the GNOME User's Guide installed on your system."),
-				 GNOME_MESSAGE_BOX_ERROR,
-				 _("Close"), NULL);
-    
-    gtk_widget_show(mbox);
-  }
-
+  gnome_help_display_with_doc_id (gnome_program_get (), "users-guide", "gccdesktop.html", "#GCCTHEME", NULL);
 }
 static void
 click_try(GtkWidget *widget, gpointer data)
@@ -379,12 +376,12 @@ click_try(GtkWidget *widget, gpointer data)
 
 /*  if (current_theme == current_global_theme)
     return;*/
-  widget = current_theme;
-  if (!widget) 
-    return;
+  if (!current_theme)
+	  return;
+
   current_global_theme = current_theme;
-  rc = (gchar *)gtk_object_get_data(GTK_OBJECT(widget), "rc");
-  dir = (gchar *)gtk_object_get_data(GTK_OBJECT(widget), "dir");
+  rc = current_theme->rc;
+  dir = current_theme->dir;
 
   /* hack for enlightenment only!!!! */
   /* FIXME: restart what ever windowmanager you have! */
@@ -400,37 +397,40 @@ click_try(GtkWidget *widget, gpointer data)
     {
       use_theme(rc, NULL);
     }
-  gdk_error_warnings = 0;
-  signal_apply_theme(widget);
+  gdk_error_trap_push ();
+  signal_apply_theme(capplet_widget);
   gdk_flush();
   /* system(cmd); */
-  gdk_error_warnings = 1;
+  gdk_error_trap_pop ();
 }
 static void
 click_ok(GtkWidget *widget, gpointer data)
 {
+  GConfClient *client = gconf_client_get_default ();
+  
   click_try (widget, data);
-  gnome_config_set_bool ("/theme-switcher-capplet/settings/auto",GTK_TOGGLE_BUTTON (auto_preview)->active);
-  gnome_config_set_string ("/theme-switcher-capplet/settings/theme", gtk_object_get_data (GTK_OBJECT (current_theme), "name"));
-  gnome_config_set_bool ("/theme-switcher-capplet/settings/use_theme_font",
-			 GTK_TOGGLE_BUTTON (font_cbox)->active);
-  gnome_config_set_string ("/theme-switcher-capplet/settings/font",
-			   gnome_font_picker_get_font_name (GNOME_FONT_PICKER (font_sel)));
-  gnome_config_sync ();
+
+  gconf_client_set_bool (client, "/apps/gtk-theme-switcher/auto", GTK_TOGGLE_BUTTON (auto_preview)->active, NULL);
+  gconf_client_set_string (client, "/apps/gtk-theme-switcher/theme", current_theme->name, NULL);
+  gconf_client_set_bool (client, "/apps/gtk-theme-switcher/use_theme_font",
+			 GTK_TOGGLE_BUTTON (font_cbox)->active, NULL);
+
+  gconf_client_set_string (client, "/apps/gtk-theme-switcher/font",
+			   gnome_font_picker_get_font_name (GNOME_FONT_PICKER (font_sel)), NULL);
+  g_object_unref (G_OBJECT (client));
 }
 static void
 click_revert(GtkWidget *widget, gpointer data)
 {
   gchar *rc;
 
-  widget = initial_theme;
-  if (!widget)
+  if (!initial_theme)
     /* we hope this doesn't happen, but it could if things
      * are mis-installed -jrb */
     /* Damn, I hate this code... )-: */
     return;
   
-  rc = (gchar *)gtk_object_get_data(GTK_OBJECT(widget), "rc");
+  rc = initial_theme->rc;
     
   if ((current_global_theme != initial_theme) ||
 	(initial_font_cbox != GTK_TOGGLE_BUTTON (font_cbox)->active) ||
@@ -443,12 +443,12 @@ click_revert(GtkWidget *widget, gpointer data)
        * or if the theme has been set.. */
       send_reread();
       use_theme(rc, initial_font);
-      gdk_error_warnings = 0;
+      gdk_error_trap_push ();
       signal_apply_theme(widget);
       gdk_flush();
-      gdk_error_warnings = 1;
+      gdk_error_trap_pop ();
     }
-  current_global_theme = widget;
+  current_global_theme = initial_theme;
   ignore_change = TRUE;
   gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (auto_preview),
 			       initial_preview);
@@ -458,7 +458,7 @@ click_revert(GtkWidget *widget, gpointer data)
   if (initial_font)
     gnome_font_picker_set_font_name (GNOME_FONT_PICKER (font_sel),
 				     initial_font);
-  gtk_list_select_child (GTK_LIST (theme_list), initial_theme);
+  gtk_clist_select_row (GTK_CLIST (theme_list), initial_theme->row, -1);
   test_theme(rc, initial_font);
   send_reread();
   if (!GTK_TOGGLE_BUTTON (font_cbox)->active)
@@ -468,14 +468,11 @@ click_revert(GtkWidget *widget, gpointer data)
   ignore_change = FALSE;
   current_theme = initial_theme;
 }
-static void
-click_entry(GtkWidget *widget, gpointer data)
-{
-  gchar *rc, *name;
-  
-  name = (gchar *)gtk_object_get_data(GTK_OBJECT(widget), "name");
-  rc = (gchar *)gtk_object_get_data(GTK_OBJECT(widget), "rc");
 
+static void
+click_entry(GtkWidget *clist, gint row, gint col, GdkEvent *event,
+	    gpointer data)
+{
   /* Load in the README file */
 #if 0
   if (readme_current)
@@ -501,120 +498,103 @@ click_entry(GtkWidget *widget, gpointer data)
 #endif
   if (!ignore_change)
     {
-      current_theme = widget;
-
-      if (initial_theme)
-	capplet_widget_state_changed(CAPPLET_WIDGET (capplet_widget), TRUE);
-      else
-	capplet_widget_state_changed(CAPPLET_WIDGET (capplet_widget), FALSE);
+      current_theme = gtk_clist_get_row_data (GTK_CLIST (clist), row);
+      
+      state_changed (TRUE);
       
       if (GTK_TOGGLE_BUTTON (auto_preview)->active)
-	click_preview (widget,NULL);
+	click_preview (NULL,NULL);
     }
 }
 
 static void
-delete_entry(GtkWidget *widget, gpointer data)
+item_destroy_notify (gpointer data)
 {
-  gchar *rc, *name, *icon, *dir;
+  ThemeEntry *item = data;
 
-  name = (gchar *)gtk_object_get_data(GTK_OBJECT(widget), "name");
-  rc = (gchar *)gtk_object_get_data(GTK_OBJECT(widget), "rc");
-  dir = (gchar *)gtk_object_get_data(GTK_OBJECT(widget), "dir");
-  icon = (gchar *)gtk_object_get_data(GTK_OBJECT(widget), "icon");
-  g_free(name);
-  g_free(rc);
-  g_free(dir);
-  g_free(icon);
+  g_free(item->name);
+  g_free(item->rc);
+  g_free(item->dir);
+  g_free(item->icon);
   
-  if (current_theme == widget)
+  if (current_theme == item)
     current_theme = NULL;
+
+  g_free (item);
 }
 static gint sort_alpha(const void *a, const void *b)
 {
-  GtkBin *A, *B;
+  const ThemeEntry *A, *B;
 
-  A = GTK_BIN (a);
-  B = GTK_BIN (b);
+  A = a;
+  B = b;
   
-  return strcmp((char *)GTK_LABEL (A->child)->label, (char *)GTK_LABEL (B->child)->label);
+  return g_strcasecmp(A->name, B->name);
+}
+
+static void
+add_theme_list (GtkWidget *disp_list, GList *themes, gchar *d_theme, gchar *current_name)
+{
+  ThemeEntry *item;
+  GList *l;
+
+  for (l = themes; l != NULL; l = l->next)
+    {
+      gchar *text[1] = { NULL };
+     
+      item = l->data;
+      text[0] = item->name;
+      item->row = gtk_clist_append (GTK_CLIST(disp_list), text);
+
+      gtk_clist_set_row_data_full (GTK_CLIST(disp_list), item->row, item,
+		      		   item_destroy_notify);
+
+      if (strcmp (d_theme, item->name) == 0)
+	{
+	  current_global_theme = item;
+	  initial_theme = item;
+	}
+      if (current_name && (strcmp (current_name, item->name) == 0))
+	{
+	  current_theme = item;
+	}
+    }
 }
 
 void
 update_theme_entries(GtkWidget *disp_list)
 {
-  ThemeEntry *te;
-  gint         num;
-  GList      *list;
-  int         i;
-  GtkWidget  *item;
-  gchar *d_theme = gnome_config_get_string ("/theme-switcher-capplet/settings/theme=Default");
+  GList      *themes;
+  GConfClient *client = gconf_client_get_default ();
+
+  gchar *d_theme = gconf_client_get_string (client, "/apps/gtk-theme-switcher/theme", NULL);
   gchar *current_name = NULL;
 
+  g_object_unref (G_OBJECT (client));
+
   if (current_theme)
-    current_name = g_strdup(gtk_object_get_data(GTK_OBJECT(current_theme), "name"));
+    current_name = g_strdup (current_theme->name);
   else
     current_name = d_theme;
 
   current_theme = NULL;
   initial_theme = NULL;
-  
-  list = NULL;
-  gtk_list_clear_items(GTK_LIST(disp_list), 0, -1);
-  te = list_system_themes(&num);
-  for (i = 0; i < num; i++)
-    {
-      item = gtk_list_item_new_with_label(te[i].name);
-      gtk_widget_show(item);
-      if (strcmp (d_theme, te[i].name) == 0)
-	{
-	  current_global_theme = item;
-	  initial_theme = item;
-	}
-      if (current_name && (strcmp (current_name, te[i].name) == 0))
-	{
-	  current_theme = item;
-	}
-      
-      gtk_object_set_data(GTK_OBJECT(item), "name", g_strdup(te[i].name));
-      gtk_object_set_data(GTK_OBJECT(item), "rc", g_strdup(te[i].rc));
-      gtk_object_set_data(GTK_OBJECT(item), "dir", g_strdup(te[i].dir));
-      gtk_object_set_data(GTK_OBJECT(item), "icon", g_strdup(te[i].icon));
-      gtk_signal_connect(GTK_OBJECT(item), "select",
-			 GTK_SIGNAL_FUNC(click_entry), NULL);
-      gtk_signal_connect(GTK_OBJECT(item), "destroy",
-			 GTK_SIGNAL_FUNC(delete_entry), NULL);
-      list = g_list_insert_sorted(list, item, sort_alpha);
-    }
-  free_theme_list(te, num);
-
-  te = list_user_themes(&num);
-  for (i = 0; i < num; i++)
-    {
-      item = gtk_list_item_new_with_label(te[i].name);
-      gtk_widget_show(item);
-      if (strcmp (d_theme, te[i].name) == 0)
-	initial_theme = item;
-      if (current_name && (strcmp (current_name, te[i].name) == 0))
-	current_theme = item;
-
-      gtk_object_set_data(GTK_OBJECT(item), "name", g_strdup(te[i].name));
-      gtk_object_set_data(GTK_OBJECT(item), "rc", g_strdup(te[i].rc));
-      gtk_object_set_data(GTK_OBJECT(item), "dir", g_strdup(te[i].dir));
-      gtk_object_set_data(GTK_OBJECT(item), "icon", g_strdup(te[i].icon));
-      gtk_signal_connect(GTK_OBJECT(item), "select",
-			 GTK_SIGNAL_FUNC(click_entry), NULL);
-      gtk_signal_connect(GTK_OBJECT(item), "destroy",
-			 GTK_SIGNAL_FUNC(delete_entry), NULL);
-      list = g_list_insert_sorted(list, item, sort_alpha);
-    }
-  free_theme_list(te, num);
-
+ 
   /* Suppress an update here, because the BROWSE mode will
    * cause a false initial selection
    */
   ignore_change = TRUE;
-  gtk_list_append_items(GTK_LIST(disp_list), list);
+ 
+  gtk_clist_clear (GTK_CLIST(disp_list));
+  
+  themes = list_system_themes();
+  themes = g_list_sort (themes, sort_alpha);
+  add_theme_list (disp_list, themes, d_theme, current_name);
+
+  themes = list_user_themes();
+  themes = g_list_sort (themes, sort_alpha);
+  add_theme_list (disp_list, themes, d_theme, current_name);
+
   ignore_change = FALSE;
 
   if (!current_theme)
@@ -626,15 +606,15 @@ update_theme_entries(GtkWidget *disp_list)
   if (current_theme)
     {
       if (current_name &&
-	  strcmp (gtk_object_get_data(GTK_OBJECT(current_theme), "name"),
+	  strcmp (current_theme->name,
 		  current_name) != 0)
 	{
-	  gtk_list_select_child (GTK_LIST (disp_list), current_theme);
+	  gtk_clist_select_row (GTK_CLIST (disp_list), current_theme->row, 0);
 	}
       else
 	{
 	  ignore_change = TRUE;
-	  gtk_list_select_child (GTK_LIST (disp_list), current_theme);
+	  gtk_clist_select_row (GTK_CLIST (disp_list), current_theme->row, 0);
 	  ignore_change = FALSE;
 	}
     }
@@ -647,3 +627,24 @@ update_theme_entries(GtkWidget *disp_list)
   if (current_theme == NULL)
     ;
 }
+
+static void
+response_cb (GtkDialog *dialog, GtkResponseType response, gpointer data)
+{
+	switch (response)
+	{
+	case GTK_RESPONSE_NONE:
+	case GTK_RESPONSE_CLOSE:
+		gtk_main_quit ();
+		break;
+	case GTK_RESPONSE_APPLY:
+		click_ok (NULL, NULL);
+		break;
+	}
+}
+
+static void state_changed (gboolean state)
+{
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (capplet_widget),
+		  		     GTK_RESPONSE_APPLY, state);
+}
